/*
-----------------------------------------------
Game main loop and kernel
-----------------------------------------------
*/

#include "kernel.h"

#define FPSQ	30	/* timer update ratio, in BPS */

int				        game_over = FALSE;          // Game Over flag
int                     game_state = UL_CONTINUE;   // Estado do jogo, diz se est em andamento ou j terminou
volatile int	        speed_counter = 0;          // Counter for the main timer
volatile int   			gClock = 0;                 //  um bagulho que serve para sincronizar todos os componentes. Varivel mais importante do programa!
volatile int	        frame_count, fps;           // this 2 counters are for count the fps

/* Timer callback for the speed counter */
void increment_speed_counter()
{
	speed_counter++;
	gClock++;
}
END_OF_FUNCTION(increment_speed_counter);



/* Timer callback for measuring the frames per second */
void fps_proc(void)
{
	fps = frame_count;
	frame_count = 0;
}
END_OF_FUNCTION(fps_proc);


/* Kernel start up.
Here you should also init all your game, and all stuff
You may prefer to load datafiles in main.c or other place,
before comin' here.
*/
void gkernel_init()
{
	// set the timer
	LOCK_VARIABLE(speed_counter);
	LOCK_FUNCTION(increment_speed_counter);

	LOCK_VARIABLE(fps);
	LOCK_VARIABLE(frame_count);
	LOCK_FUNCTION(fps_proc);

	/* install the timers */
	if (install_int_ex(increment_speed_counter, BPS_TO_TIMER(FPSQ)))
		raise_error("gkernel_init() : Can't install timer at %d bps\n", FPSQ);
	if (install_int(fps_proc, 1000))
		raise_error("gkernel_init() : Can't install timer to measure fps.");

	/* here start all the variables and all stuff */
	CCenario *cenario;
	cenario = new CCenario(&gClock);
	cenario->Init();


	game_over = FALSE;
	frame_count = fps = gClock = 0;

	/* Main game loop */
	while (!game_over)
	{

		while (speed_counter > 0)
		{
		    game_state = gkernel_update_logic(cenario);
			if (game_state == UL_PLAYER_EXIT || game_state == UL_PLAYER_WIN)
                game_over = TRUE;
			speed_counter--;
		}
		gkernel_update_screen(cenario);
		frame_count++; // count fps
	}
	remove_int(increment_speed_counter);
	remove_int(fps_proc);
}



// Updates the game logic.
int gkernel_update_logic(CCenario *cenario)
{
    static unsigned int tempo;
    if(gClock - tempo > 30)
	{
		cenario->AtualizaPosicaoTrem();
		tempo = gClock;
	}

	int tecla = 0;
    while (keypressed())
        tecla = readkey();

	poll_mouse();
    if (mouse_b & 2)
    {
		while (mouse_b  & 2)
            poll_mouse();
        cenario->MudaTrilho();
        mouse_b = 0;
	}
	if ((mouse_b & 1) && mouse_x < 480 && mouse_y < 480)
	{
        while (mouse_b  & 1)
            poll_mouse();
		cenario->PoeTrilho(mouse_y / 32, mouse_x / 32);
	}

    switch(tecla >> 8)
    {
    case KEY_ESC:
        return UL_PLAYER_EXIT;
        break;
    }

    return UL_CONTINUE;
}



// Updates the graphics
void gkernel_update_screen(CCenario *cenario)
{
    static BITMAP *buffer = create_bitmap(SCREEN_W, SCREEN_H);

    switch(game_state)
    {
    case UL_CONTINUE:
		acquire_bitmap(buffer);
        clear_to_color(buffer, makecol(0, 0, 0));
        draw_sprite(buffer, cenario->Draw(), 0, 0);
        textprintf_ex(buffer, font, 480, 0, makecol(255, 255, 255), 0, "%5d fps", fps);
        textprintf_ex(buffer, font, 480, 16, makecol(255, 255, 255), 0, "%5d", gClock / 30);
        textprintf_ex(buffer, font, 480, 24, makecol(255, 255, 255), 0, "tempo = %2d", cenario->TempoRestanteMontagem() / 30);
        draw_sprite(buffer, cenario->tiles[cenario->trilhoAtual], mouse_x - 16, mouse_y - 16);
        release_bitmap(buffer);
        break;

    case UL_PLAYER_EXIT:
        clear_to_color(screen, 0);
        textout_ex(screen, font, "realmente, muito difcil.", 0, 0, makecol(255, 255, 255), 0);
        clear_keybuf();
        readkey();
        break;

    case UL_PLAYER_WIN:
        clear_to_color(screen, 0);
        textout_ex(screen, font, "Parabns, voc venceu!", 0, 0, makecol(255, 255, 255), 0);
        readkey();
        break;
    }
    blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
}
